/*
Copyright (c) 2010-2011 The Regents of the University of California. All rights
reserved.

Permission is hereby granted, without written agreement and without license or 
royalty fees, to use, copy, modify, and distribute this software and its 
documentation for any purpose, provided that the above copyright notice and the
following two paragraphs appear in all copies of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF 
CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

/**
 * @file ptp-config.h
 * Define the configurable constants that are required for the correct 
 * operation of PTP.
 *
 * @author Michael Zimmer (mzimmer@eecs.berkeley.edu)
 * @version $Id: ptp-config.h 50 2011-04-11 23:39:21Z mzimmer $
 */

#define LOGANNOUNCEINTERVAL 1 /**< Value for the PortDS.logAnnounceInterval 
* member of PTPState.portDS. 
*/
#define LOGSYNCINTERVAL 0 /**< Value for the PortDS.logSyncInterval member
* of PTPState.portDS.
*/
#define LOGMINDELAYREQINTERVAL 0 /**< Value for the
* PortDS.logMinDelayReqInterval member of PTPState.portDS.
*/
#define DELAYREQDIV 4 /**< 2^DELAYREQDIV is the number of possible values
* (granularity) in the range of 2^(LOGMINDELAYREQINTERVAL+1) that can be set as
* the transmission interval for a Delay_Req message (9.5.11.2). To ensure
* adequate granularity, DELAYREQDIV >= LOGMINDELAYREQINTERVAL+4.
*/
#define ANNOUNCERECEIPTTIMEOUT 3 /**< Value for the
* PortDS.announceReceiptTimeout member of PTPState.portDS. */

// DefaultDS (8.2.1.2)
#define TWOSTEPFLAG FALSE /**< Value for the DefaultDS.twoStepFlag member of
* PTPState.defaultDS. 
*/ 
#define DOMAINNUMBER 0 /**< Value for the DefaultDS.domainNumber member of
* PTPState.defaultDS. The default is 0. 
*/ 
#define CLOCKIDENTITY {0x5C, 0xFF, 0x35, 0xFF, 0xFE, 0x0D, 0x01, 0x4F} /**< The
* value for the DefaultDS.clockIdentity member of PTPState.defaultDS. For using
* a EUI-48 based MAC address, the clock identity is defined as {first 3 bytes of
* MAC, 0xFF, 0xFE, last 3 bytes of MAC}.
*/
#define NUMBERPORTS 1 /**< Value for the DefaultDS.numberPorts member of
* PTPState.defaultDS. The value is 1 for an ordinary clock.
*/
#define CLOCKCLASS 248 /**< Value for the ClockQuality.clockClass member 
* of the DefaultDS.clockQuality member of PTPState.defaultDS. The default is
* 248. 
*/
#define CLOCKACCURACY 0xFE /**< Value for the ClockQuality.clockAccuracy
* member of the DefaultDS.clockQuality member of PTPState.defaultDS. The default
* is 0xFE (unknown). 
*/
#define OFFSETSCALEDLOGVARIANCE 0xFFFF  /**< Value for the 
* ClockQuality.offsetScaledLogVareiance member of the DefaultDS.clockQuality 
* member of PTPState.defaultDS. If unknown, the value is 0xFFFF. TODO: Need to
* measure (7.6.3.2).
*/
#define PRIORITY1 136 /**< Value for the DefaultDS.priority1 member of
* PTPState.defaultDS. 
*/ 
#define PRIORITY2 128 /**< Value for the DefaultDS.priority2 member of
* PTPState.defaultDS. 
*/ 
#define SLAVEONLY FALSE /**< Value for the DefaultDS.slaveOnly member of
* PTPState.defaultDS.
*/

// TimePropertiesDS (8.2.4)
#define CURRENTUTCOFFSET 34 /**< Value for the
* TimePropertiesDS.currentUtcOffset member of PTPState.timePropertiesDS.
*/
#define CURRENTUTCOFFSETVALID TRUE /**< Value for the 
* TimePropertiesDS.currentUtcOffsetValid member of PTPState.timePropertiesDS.
*/
#define LEAP59 FALSE/**< Value for the TimePropertiesDS.leap59 member
* of PTPState.timePropertiesDS.
*/
#define LEAP61 FALSE/**< Value for the TimePropertiesDS.leap61 member
* of PTPState.timePropertiesDS.
*/
#define TIMETRACEABLE FALSE/**< Value for the TimePropertiesDS.timeTraceable
* member of PTPState.timePropertiesDS.
*/
#define FREQUENCYTRACEABLE FALSE/**< Value for the 
* TimePropertiesDS.frequencyTraceable member of PTPState.timePropertiesDS.
*/
#define PTPTIMESCALE TRUE/**< Value for the TimePropertiesDS.ptpTimescale
* member of PTPState.timePropertiesDS.
*/
#define TIMESOURCE 0xA0  /**< Value for the TimePropertiesDS.timeSource 
* member of PTPState.timePropertiesDS. An internal oscillator is 0xA0.
*/

#define FOREIGNMASTERDSSIZE 5 /**< Array size of PTPState.foreignMasterDS
* (9.3.2.4.5). The minimum value is 5. 
*/

#define DELAY_ASYMMETRY 0 /**< Asymmetry in communication path in units
* of nanoseconds multiplied by 2^16 (7.4.2).
*/

#define MEANPATHDELAY 0x27100000 /**< Initialization value of mean path delay,
* in units of nanoseconds multiplied by 2^16. Either use 0 or best estimate of
* mean path delay.
*/

#define CUTOFF 100000 /**< Cutoff for deciding whether to step the clock or
* adjust the frequency, in units of ns.  The value used is
* implementation specific since it depends on the importance of not stepping 
* the clock, the maximal rate adjustment allowed, and constraints on the time 
* to lock. For the DP83640, rate adjustment can achieve ~1950ns/sec, so a cutoff
* of 100,000ns could have a time to lock of ~51 seconds.
*/

#define OFFSET_P(x) 0.2*(x) /**< Proportional term of the PI 
* controller which operates on the error in clock freqency between master 
* and slave clocks.
*/
#define OFFSET_I(x) 0*(x) /**< Integral term of the PI 
* controller which operates on the error in clock freqency between master 
* and slave clocks.
*/
#define FREQ_P(x) 0.2*(x) /**< Proportional term of the PI 
* controller which operates on the error in clock offset between master 
* and slave clocks.
*/
#define FREQ_I(x) 0*(x) /**< Integral term of the PI 
* controller which operates on the error in clock offset between master 
* and slave clocks.
*/

// Annex D. PTP over UDP over IP.
#define PTP_EVENT_PORT 319 /**< PTP event port (Annex D). */
#define PTP_GENERAL_PORT 320 /**< PTP general port (Annex D). */
#define DEST_IP {0xE000, 0x0181} /**< Destination IP address (Annex D). */
#define TRANSPORTSPECIFIC 0 /**< Value of Header.transportSpecific 
* (Annex D).
*/
#define ADDRESSLENGTH /**< Address length (Annex D). */

